PROGRAM TOF_Model
IMPLICIT NONE
INTEGER                       :: i, j, k, n, unit
INTEGER                       :: steps, step_sum, coord_max_m(2), TDI_step, TDTS_step, n_effreac, n_effprod
INTEGER                       :: n_effreac_real, n_effprod_real
INTEGER, allocatable          :: indices_effreac(:), indices_effprod(:)
INTEGER, allocatable          :: rate_indices(:,:,:), im_indices(:,:,:), ts_indices(:,:,:), im_rem(:,:), ts_rem(:,:)
DOUBLE PRECISION              :: deltag, deltag_kcal, temperature, concproduct_reac, concproduct_prod, factor
DOUBLE PRECISION              :: TOF, numerator, denominator
DOUBLE PRECISION, allocatable :: matrix_energy(:,:), matrix_excel(:,:), matrix_conc(:,:), matrix_complete(:,:)
DOUBLE PRECISION, allocatable :: TS(:), IM(:), TS_kcal(:), IM_kcal(:), conc_reac(:), conc_prod(:), X_TOF_I(:), X_TOF_TS(:)
DOUBLE PRECISION, allocatable :: sum_reac(:), X_TOF_reac(:), sum_prod(:), X_TOF_prod(:)
DOUBLE PRECISION              :: const_k, const_NA, const_R, const_h, const_caloryTOjoule
CHARACTER*20                  :: steps_dependent_format_E, steps_dependent_format_I, TDI_step_char, TDTS_step_char
CHARACTER*3, allocatable      :: indices_reac_char(:), indices_prod_char(:)
CHARACTER*100                 :: inputfilename, teststring, stringtest
CHARACTER*20                  :: outputoptions(5)
CHARACTER*100,allocatable     :: conc_prod_string(:), conc_reac_string(:)
CHARACTER*9,allocatable       :: X_reac_string(:), X_prod_string(:)
LOGICAL                       :: debug_energy, debug_rate_indices, debug_im_indices, debug_ts_indices, debug_im_rem
LOGICAL                       :: debug_ts_rem, debug_matrix_energy, debug_matrix_excel, debug_matrix_conc
LOGICAL                       :: debug_matrix_complete, debug_numer_denom, debug_XTOFC
LOGICAL                       :: tester, fileexists, out_input, out_degrees, out_analysis, out_tof, out_header 
LOGICAL                       :: out_licence
LOGICAL,allocatable           :: reac_exist(:), prod_exist(:)

! set debug flags
debug_energy          = .false.
debug_rate_indices    = .false.
debug_im_indices      = .false.
debug_ts_indices      = .false.
debug_im_rem          = .false.
debug_ts_rem          = .false.
debug_matrix_energy   = .false.
debug_matrix_excel    = .false.
debug_matrix_conc     = .false.
debug_matrix_complete = .false.
debug_numer_denom     = .false.
debug_XTOFC           = .false.

! declare some constants; unit is given in comment above
! Boltzmann constant in J/K
const_k  = 1.380658E-23
! Avogadro constant in 1/mol
const_NA = 6.0221367E+23
! universal gas constant in J/(mol*K)
const_R = const_k * const_NA
! Planck's constant in J*s
const_h  = 6.6260755E-34
! factor to convert calories to joules in J/cal
const_caloryTOjoule = 4.184

! I/O unit of input file
unit=99

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!! READ INPUT SECTION !!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! get name of the input file
CALL GETARG(1,inputfilename)
INQUIRE(FILE=TRIM(inputfilename),EXIST=fileexists)
IF (fileexists) THEN
 OPEN(UNIT=unit,FILE=TRIM(inputfilename),STATUS='OLD',ACCESS='SEQUENTIAL',FORM='FORMATTED')
ELSE
 PRINT*,'The specified input file does not exist.'
 PRINT*,'Program usage: AUTOF.exe inputfile [options]'
 CALL EXIT
END IF

! get options for restricted output
DO i = 1, 5
 CALL GETARG(i + 1,outputoptions(i))
END DO
out_input    = .TRUE.
out_degrees  = .TRUE.
out_analysis = .TRUE.
out_tof      = .TRUE.
out_header   = .TRUE.
out_licence  = .TRUE.
DO j = 1, 5
 DO i = 1, 20
  IF (outputoptions(j)(i:i) .EQ. 'i') THEN
   out_input = .FALSE.
  END IF
  IF (outputoptions(j)(i:i) .EQ. 'd') THEN
   out_degrees = .FALSE.
  END IF
  IF (outputoptions(j)(i:i) .EQ. 'a') THEN
   out_analysis = .FALSE.
  END IF
  out_analysis = .FALSE.
  IF (outputoptions(j)(i:i) .EQ. 't') THEN
   out_tof = .FALSE.
  END IF
  IF (outputoptions(j)(i:i) .EQ. 'h') THEN
   out_header = .FALSE.
  END IF
  IF (outputoptions(j)(i:i) .EQ. 'l') THEN
   out_licence = .FALSE.
  END IF
 END DO
END DO

! read temperature
read(unit,*) temperature

! read number of steps in the cycle
! one step = intermediate and following transition state
read(unit,*) steps

! create formats to print arrays for debugging
write(steps_dependent_format_E,*) '(', steps, 'E10.3', ')'
write(steps_dependent_format_I,*) '(', steps, 'I3', ')'

! create arrays according to the size of the cycle
allocate(TS(steps))
allocate(IM(0:2*steps-1))
allocate(TS_kcal(steps))
allocate(IM_kcal(0:2*steps-1))
allocate(conc_reac(steps))
allocate(conc_prod(steps))
allocate(rate_indices(steps,steps,steps-1))
allocate(ts_indices(steps,steps,steps-1))
allocate(im_indices(steps,steps,steps-1))
allocate(ts_rem(steps,steps))
allocate(im_rem(steps,steps))
allocate(matrix_energy(steps,steps))
allocate(matrix_excel(steps,steps))
allocate(matrix_conc(steps,steps))
allocate(matrix_complete(steps,steps))
allocate(X_TOF_I(0:steps-1))
allocate(X_TOF_TS(steps))
allocate(sum_reac(steps))
allocate(sum_prod(steps))
allocate(X_TOF_reac(steps))
allocate(X_TOF_prod(steps))
allocate(reac_exist(steps))
allocate(prod_exist(steps))
allocate(conc_reac_string(steps))
allocate(conc_prod_string(steps))
allocate(X_reac_string(steps))
allocate(X_prod_string(steps))

! read in values for intermediate energy, TS energy, reactant
! concentration, product concentration
! file with values can be created with Excel saving in .txt format
DO i = 1, steps
 read(unit,*) IM_kcal(i-1), TS_kcal(i), conc_reac_string(i), conc_prod_string(i)
END DO

! read in deltag
read(unit,*) deltag_kcal
CLOSE(UNIT=unit)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!! DATA PROCESSING SECTION !!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! determine where reactants or products exist and what their initial
! concentration is
DO i = 1, steps
 IF (TRIM(conc_reac_string(i)) .EQ. 'x' .OR. TRIM(conc_reac_string(i)) .EQ. 'X') THEN
  reac_exist(i) = .false.
  conc_reac(i) = 1
  conc_reac_string(i)='x'
 ELSE
  reac_exist(i) = .true.
  read(conc_reac_string(i),*) conc_reac(i)
  write(conc_reac_string(i),'(ES7.1)') conc_reac(i)
 END IF 
 IF (TRIM(conc_prod_string(i)) .EQ. 'x' .OR. TRIM(conc_prod_string(i)) .EQ. 'X') THEN
  prod_exist(i) = .false.
  conc_prod(i) = 1
  conc_prod_string(i)='x'
 ELSE
  prod_exist(i) = .true.
  read(conc_prod_string(i),*) conc_prod(i)
  write(conc_prod_string(i),'(ES7.1)') conc_prod(i)
 END IF 
END DO

! calculate the intermediate energies of the second cycle
! they differ from the first cycle by deltag
DO i = steps, 2 * steps - 1
 IM_kcal(i)= IM_kcal(i - steps) + deltag_kcal
END DO

! convert energies from kcal/mol to dimensionless figures in k_B*T units 
! as in S. Shaik, S. Kozuch, JACS 2006, 128, 3355:
! explained between equation (8) and (9)
factor = const_caloryTOjoule * 1000 / (const_R * temperature)
deltag = deltag_kcal * factor
DO i = 1, steps
 TS(i) = TS_kcal(i) * factor
END DO
DO i = 0, 2 * steps - 1
 IM(i) = IM_kcal(i) * factor
END DO

! calculate numerator as in
! S. Kozuch, S. Shaik, J. Phys. Chem. A 2008, 112, 6032:
! equation (a17)
concproduct_reac = 1
concproduct_prod = 1
DO i = 1, steps
 concproduct_reac = concproduct_reac * conc_reac(i)
 concproduct_prod = concproduct_prod * conc_prod(i)
END DO
numerator = exp(-deltag) * concproduct_reac - concproduct_prod

IF (numerator .LE. 0) THEN
 print*,
 print*, 'WARNING! There is no positive driving force.'
 IF (deltag .GE. 0) THEN
  print*, "That's because DeltaG is not negative."
 ELSE
  print*, 'Maybe you should modify the concentrations.'
 END IF
!  print*, 'The program is going to exit.'
!  deallocate(TS)
!  deallocate(IM)
!  deallocate(TS_kcal)
!  deallocate(IM_kcal)
!  deallocate(conc_reac)
!  deallocate(conc_prod)
!  deallocate(rate_indices)
!  deallocate(ts_indices)
!  deallocate(im_indices)
!  deallocate(ts_rem)
!  deallocate(im_rem)
!  deallocate(matrix_energy)
!  deallocate(matrix_excel)
!  deallocate(matrix_conc)
!  deallocate(matrix_complete)
!  deallocate(X_TOF_I)
!  deallocate(X_TOF_TS)
!  deallocate(sum_reac)
!  deallocate(sum_prod)
!  deallocate(X_TOF_reac)
!  deallocate(X_TOF_prod)
!  deallocate(reac_exist)
!  deallocate(prod_exist)
!  deallocate(conc_reac_string)
!  deallocate(conc_prod_string)
!  deallocate(X_reac_string)
!  deallocate(X_prod_string)
!  CALL EXIT
END IF

! create an array holding the indices of the rate constants according to
! the matrix M in Christiansen, J.A., Adv. Catal. 1953, 5, 311:
! page 333
!
! first index of array:  line index of matrix
! second index of array: column index of matrix
! third index of array:  distinguishing the indices of the rate
! constants within one term of the matrix
!
! start with k_2 as it is said in the paper
rate_indices=0
rate_indices(1,1,1)=2
! create first entry
DO i = 2, steps - 1
 rate_indices(1,1,i) = rate_indices(1,1,i-1) + 1
END DO
! derive first column from first entry
DO i = 2, steps
 DO j = 1, steps - 1
  rate_indices(i,1,j) = rate_indices(i-1,1,j) + 1 - int(rate_indices(i-1,1,j)/steps) * steps
 END DO
END DO
! derive the other columns from the first column
DO k = 1, steps
 DO j = 2, steps
  DO i = 1, steps-1
   IF (j > i) THEN
    rate_indices(k,j,i) = (rate_indices(k,1,i) + steps - 1 - int((rate_indices(k,1,i)+ steps - 2)/steps) * steps) * (-1)
   ELSE
    rate_indices(k,j,i) = rate_indices(k,1,i)
   END IF
  END DO
 END DO
END DO

! create a similar matrix with the indices of the corresponding
! intermediates instead of the indices of the rate constants
DO i = 1, steps
 DO j = 1, steps
  DO k = 1, steps - 1
   IF (rate_indices(i,j,k) < 0) THEN
    im_indices(i,j,k) = abs(rate_indices(i,j,k))
   ELSE
    im_indices(i,j,k) = rate_indices(i,j,k) - 1
   END IF
  END DO
 END DO
END DO

! create a similar matrix with the indices of the corresponding
! transitions states instead of the indices of the rate constants
DO i = 1, steps
 DO j = 1, steps
  DO k = 1, steps - 1
   ts_indices(i,j,k) = abs(rate_indices(i,j,k))
  END DO
 END DO
END DO

! get the index of the remaining intermediate energy for each term as in
! S. Kozuch, S. Shaik, J. Phys. Chem. A 2008, 112, 6032:
! equation (a8)
! implicit in (a8) is: I_(i) + deltag = I_(i + steps)
! remember:
! the order of the terms in the array corresponds to the matrix M in
! Christiansen, J.A., Adv. Catal. 1953, 5, 311:
! page 333 
step_sum = 0
im_rem = 0
DO i = 1, steps
 step_sum = step_sum + i
END DO
DO i = 1, steps
 DO j = 1, steps
  DO k = 1, steps -1
   im_rem(i,j) = im_rem(i,j) + im_indices(i,j,k)
  END DO
  im_rem(i,j) = step_sum - im_rem(i,j)
 END DO
END DO

! get the index of the remaining transition state energy for each term
! as in S. Kozuch, S. Shaik, J. Phys. Chem. A 2008, 112, 6032:
! equation (a8)
! remember:
! the order of the terms in the array corresponds to the matrix M in
! Christiansen, J.A., Adv. Catal. 1953, 5, 311:
! page 333 
ts_rem = 0
DO i = 1, steps
 DO j = 1, steps
  DO k = 1, steps - 1
   ts_rem(i,j) = ts_rem(i,j) + ts_indices(i,j,k)
  END DO
  ts_rem(i,j) = step_sum - ts_rem(i,j)
 END DO
END DO

! create the energy based matrix M as in:
! S. Kozuch, S. Shaik, J. Phys. Chem. A 2008, 112, 6032:
! equation (a8) (without the prefactor) 
! remember:
! the order of the terms in the array corresponds to the matrix M in
! Christiansen, J.A., Adv. Catal. 1953, 5, 311:
! page 333 
DO i = 1, steps
 DO j = 1, steps
   matrix_energy(i,j) = exp(TS(ts_rem(i,j))-IM(im_rem(i,j)))
 END DO
END DO

! create matrix with the concentration terms of the denominator in
! S. Kozuch, S. Shaik, J. Phys. Chem. A 2008, 112, 6032:
! equation (a17)
! remember:
! the order of the terms in the array corresponds to the matrix M in
! Christiansen, J.A., Adv. Catal. 1953, 5, 311:
! page 333 
matrix_conc = 1
DO i = 1, steps
 DO j = 1, steps
  DO k = 1, steps-1
   IF (rate_indices(i,j,k) > 0) THEN
    matrix_conc(i,j) = matrix_conc(i,j) * conc_reac(rate_indices(i,j,k))
   ELSE
    matrix_conc(i,j) = matrix_conc(i,j) * conc_prod(abs(rate_indices(i,j,k)))
   END IF
  END DO
 END DO
END DO

! create matrix holding the single summation terms (energy and
! concentration) of the denominator in
! S. Kozuch, S. Shaik, J. Phys. Chem. A 2008, 112, 6032.
! equation (a17)
matrix_complete = 1
DO i = 1, steps
 DO j = 1, steps
    matrix_complete(i,j) = matrix_energy(i,j) * matrix_conc(i,j)
 END DO
END DO

! calculate denominator by summing over the elements of  as in
! S. Kozuch, S. Shaik, J. Phys. Chem. A 2008, 112, 6032:
! equation (a17)
denominator = 0
DO i = 1, steps
 DO j = 1, steps
  denominator = denominator + matrix_complete(i,j)
 END DO
END DO

! calculate the TOF
factor = const_k * temperature /const_h
TOF = factor * numerator / denominator 

! find the highest value in M and find out which intermediate and
! transition state go into it; these are the TDI and TDTS
! remember: the index of the intermediate is by one lower than the step
! it belongs to
coord_max_m = maxloc(matrix_complete)
TDI_step  = im_rem(coord_max_m(1),coord_max_m(2)) + 1 - int(im_rem(coord_max_m(1),coord_max_m(2)) / steps) * steps
TDTS_step = ts_rem(coord_max_m(1),coord_max_m(2))

! calculate degree of TOF control for every intermediate and
! transition state
X_TOF_I  = 0
X_TOF_TS = 0
DO k = 1, steps
 DO i = 1, steps
  DO j = 1, steps
   IF (im_rem(i,j) .EQ. k - 1 .OR. im_rem(i,j) - steps .EQ. k - 1) THEN
    X_TOF_I(k - 1) = X_TOF_I(k - 1) + matrix_complete(i,j)
   END IF
   IF (ts_rem(i,j) .EQ. k) THEN
    X_TOF_TS(k) = X_TOF_TS(k) + matrix_complete(i,j)
   END IF
  END DO
 END DO
END DO
DO i = 1, steps
 X_TOF_I(i - 1)  = X_TOF_I(i - 1)  / denominator
 X_TOF_TS(i) = X_TOF_TS(i) / denominator
END DO

! use the result from the TOF approximation to predict which
! concentrations are important
! see S. Kozuch, S. Shaik, J. Phys. Chem. A 2008, 112, 6032:
! equation (a24) and sentence below
n_effreac = TDTS_step - (TDI_step - 1) - int((TDTS_STEP - (TDI_step - 1) - steps) / steps) * steps
n_effprod = n_effreac - 1
allocate(indices_effreac(n_effreac))
allocate(indices_effprod(n_effprod))
DO i = 1, n_effreac
 indices_effreac(i) = TDI_step + i - 1 - int((TDI_step + i - 2) / steps) * steps
END DO
DO i = 1, n_effprod
 indices_effprod(i) = indices_effreac(i)
END DO

! ignore those concentration indices where no reactant enters/product
! leaves the cycle and arrange the indices of the concentrations in
! ascending order
n_effreac_real = 0
DO i = 1, n_effreac
 IF (reac_exist(indices_effreac(i))) THEN
  n_effreac_real = n_effreac_real + 1
 ELSE
  indices_effreac(i) = steps + 1
 END IF
END DO
n_effprod_real = 0
DO i = 1, n_effprod
 IF (prod_exist(indices_effreac(i))) THEN
  n_effprod_real = n_effprod_real + 1
 ELSE
  indices_effprod(i) = steps + 1
 END IF
END DO
allocate(indices_reac_char(n_effreac_real))
allocate(indices_prod_char(n_effprod_real))
DO i = 1, n_effreac_real
 write(indices_reac_char(i),*) MINVAL(indices_effreac)
 indices_effreac(MINLOC(indices_effreac)) = steps + 1
END DO
DO i = 1, n_effprod_real
 write(indices_prod_char(i),*) MINVAL(indices_effprod)
 indices_effprod(MINLOC(indices_effprod)) = steps + 1
END DO




! calculate the degree of TOF control for every reactant and product
! concentration
! use therefor the same ansatz as for the degree of TOF control of the
! energies as in 
! S. Shaik, S. Kozuch, JACS 2006, 128, 3355:
! equation (10)
! explicitely:
! X_TOF_[R_i] = | [R_i]/TOF * dTOF / d[R_i] |
! X_TOF_[P_i] = | [P_i]/TOF * dTOF / d[P_i] |
!
! let TOF   = u / v                                        
!  => TOF'  = (u'v - uv') / v^2                            
!  => X_TOF = [C_i] * (u'v - uv') / uv                     
!           = [C_i] * (u'/u - v'/v)                        
!           = ([C_i] * u') / u - ([C_i] * v_i) / v         
!
! It can be shown that:
! u = numerator
! v = denominator
! ([R_i] * u')=numerator + concproduct_prod
! ([R_i] * v')=sum of all elements of matrix_complete that contain [R_i]
!             ->will be named sum_reac
! ([P_i] * u')= - concproduct_prod
! ([P_i] * v')=sum of all elements of matrix_complete that contain [P_i]
!             ->will be named sum_prod

! calculate degree of TOF_control for reactants
sum_reac = 0
DO n = 1, steps
 DO i = 1, steps
  DO j = 1, steps
   tester = .FALSE.
   DO k = 1, steps - 1
    tester = tester .OR. rate_indices(i,j,k) .EQ. n
   END DO
   IF (tester) THEN
    sum_reac(n) = sum_reac(n) + matrix_complete(i,j)
   END IF
  END DO
 END DO
END DO
DO i = 1, steps
 X_TOF_reac(i) = (numerator + concproduct_prod) / numerator - sum_reac(i) / denominator
END DO

! calculate degree of TOF_control for products
sum_prod = 0
DO n = 1, steps
 DO i = 1, steps
  DO j = 1, steps
   tester = .FALSE.
   DO k = 1, steps - 1
    tester = tester .OR. rate_indices(i,j,k) .EQ. -n
   END DO
   IF (tester) THEN
    sum_prod(n) = sum_prod(n) + matrix_complete(i,j)
   END IF
  END DO
 END DO
END DO
DO i = 1, steps
 X_TOF_prod(i) =  -concproduct_prod / numerator - sum_prod(i) / denominator
END DO

! put an x for the X TOF C values without actual product or reactant
DO i = 1, steps
 IF (reac_exist(i)) THEN
  write(X_reac_string(i),'(F9.2)') X_TOF_reac(i)
 ELSE
  write(X_reac_string(i),*) 'x'
 END IF
 IF (prod_exist(i)) THEN
  write(X_prod_string(i),'(F9.2)') X_TOF_prod(i)
 ELSE
  write(X_prod_string(i),*) 'x'
 END IF
END DO

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!! PRINT OUTPUT SECTION !!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
print*, ''
IF (out_licence) THEN
 print*, 'This is the AUTOF program, version 1.0, last modification 07/19/2010.'
 print*, 'Use of this program is only permitted under the AUTOF User License that'
 print*, 'can be obtained from the authors. It is the users responsibility to make'
 print*, 'sure the AUTOF User License is not violated. Later versions of the'
 print*, 'program might be available from the authors.'
 print*, ''
 print*, 'Authors:'
 print*, 'Andreas Uhe      (uhe@itmc.rwth-aachen.de)'
 print*, 'Sebastian Kozuch (sebastian.kozuch@weizmann.ac.il)'
 print*, 'Sason Shaik      (sason@yfaat.ch.huji.ac.il)'
 print*, ''
 print*, 'Results obtained with AUTOF and published in the scientific literature must'
 print*, 'reference to:'
 print*, 'a) S. Kozuch, S. Shaik, A Combined Kinetic-Quantum Mechanical Model for '
 print*, '   Assessment of Catalytic Cycles: Application to Cross-Coupling and Heck' 
 print*, '   Reactions, J. Am. Chem. Soc. 2006, 128, 3355-3365;'
 print*, 'b) S. Kozuck, S. Shaik, Kinetic-Quantum Chemical Model for Catalytic Cycles:'
 print*, '   The Haber-Bosch Process and the Effect of Reagent Concentration,' 
 print*, '   J. Phys. Chem. A 2008, 112, 6032-6041;'
 print*, 'c) A. Uhe, S. Kozuch, S. Shaik, Automatic Analysis of Computed'
 print*, '   Catalytic Cycles, J. Comp. Chem. 2010, submitted.'
 print*, '   (please check for the present state of the publication)'
 print*, ''
 print*, 'The authors disclaim all warranties on the use of this program.'
 print*, ''
END IF
print*, 'NAME OF THE INPUTFILE: ', TRIM(inputfilename)
print*,''
IF (out_input) THEN
 IF (out_header) THEN
  print*, '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
  print*, '!!                                                                !!'
  print*, '!!                     REPEAT OF THE INPUT                        !!'
  print*, '!!                                                                !!'
  print*, '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
  print*,''
 END IF
 print*, 'temperature =', temperature, 'K'
 print*, '# of steps  =', steps
 print*, ''

! print the energy values and concentration values
 print*, '             energies        concentrations'
 print*, '            Int     TS     reactant    product'
 DO i = 1, steps
  write(*,'(A5,I2,A1,2F8.1,A2,A10,A1,A10)') ' Step', i, ' ', IM_kcal(i-1), TS_kcal(i), '  ', TRIM(conc_reac_string(i)) &
, ' ', TRIM(conc_prod_string(i))
 END DO
 write(*,'(A7,F8.1,/)') ' Final ', IM_kcal(steps)
 write(*,'(A7,F8.1,/)') ' DeltaG', deltag_kcal
END IF

! print the degrees of TOF control for the energies and concentrations
IF (out_degrees) THEN
 IF (out_header) THEN
  print*, '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
  print*, '!!                                                                !!'
  print*, '!!                   THE DEGREES OF TOF CONTROL                   !!'
  print*, '!!                                                                !!'
  print*, '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
  print*,''
 END IF
 print*, '          X_TOF(energies)   X_TOF(concentrations)'
 print*, '            Int      TS       reactant  product'
 DO i = 1, steps
  write(*,'(A5,I2,2F9.2,A3,2A9)') ' Step', i, X_TOF_I(i - 1), X_TOF_TS(i), '   ',  TRIM(X_reac_string(i)), TRIM(X_prod_string(i))
 END DO
 print*,''
END IF

! name TDI, TDTS and their degree of TOF control, name influential
! concentrations
IF (out_analysis) THEN
 IF (out_header) THEN
  print*, '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
  print*, '!!                                                                !!'
  print*, '!!                     ANALYSIS OF THE CYCLE                      !!'
  print*, '!!                                                                !!'
  print*, '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
  print*,''
 END IF
 write(TDI_step_char,*) TDI_step
 write(TDTS_step_char,*) TDTS_step
 write(*,*) 'The TDI is the intermediate of step'//TRIM(TDI_step_char)//'.'
 write(*,*) 'The TDTS is the transition state of step'//TRIM(TDTS_step_char)//'.'
 print*,''

 IF (n_effreac_real .EQ. 1 .AND. n_effprod_real .EQ. 0) THEN 
  write(*,*) 'Using the reactant entering in step', TRIM(indices_reac_char(1)), ' in high concentration'
  write(*,*) 'will speed up the reaction more efficiently than manipulating the other'
  write(*,*) 'concentrations.'
 ELSE IF (n_effreac_real .EQ. 1 .AND. n_effprod_real .GT. 0) THEN 
  write(*,*) 'Using the reactant entering in step', TRIM(indices_reac_char(1)), ' in high concentration and'
 ELSE IF (n_effreac_real .EQ. 2 .AND. n_effprod_real .EQ. 0) THEN
  write(*,*) 'Using the reactants entering in steps', TRIM(indices_reac_char(1)),' and', TRIM(indices_reac_char(2)), &
             ' in high concentration'
  write(*,*) 'will speed up the reaction more efficiently than manipulating the other'
  write(*,*) 'concentrations.'
 ELSE IF (n_effreac_real .EQ. 2 .AND. n_effprod_real .GT. 0) THEN
  write(*,*) 'Using the reactants entering in steps', TRIM(indices_reac_char(1)),' and', TRIM(indices_reac_char(2)), &
             ' in high concentration and'
 ELSE IF (n_effreac_real .GT. 2 .AND. n_effprod_real .EQ. 0) THEN
  write(*,*) 'Using the reactants entering in steps',(TRIM(indices_reac_char(i)),',',i = 1, n_effreac_real-2), &
             TRIM(indices_reac_char(n_effreac_real-1)),' and', TRIM(indices_reac_char(n_effreac_real)),' in high concentration'
  write(*,*) 'will speed up the reaction more efficiently than manipulating the other'
  write(*,*) 'concentrations.'
 ELSE IF (n_effreac_real .GT. 2 .AND. n_effprod_real .GT. 0) THEN
  write(*,*) 'Using the reactants entering in steps',(TRIM(indices_reac_char(i)),',',i = 1, n_effreac_real-2), &
             TRIM(indices_reac_char(n_effreac_real-1)),' and', TRIM(indices_reac_char(n_effreac_real)),' in high concentration and'
 END IF 

 IF (n_effprod_real .EQ. 1 .AND. n_effreac_real .GT. 0) THEN 
  write(*,*) 'removing the product released in step', TRIM(indices_prod_char(1)), ' from the reaction mixture'
  write(*,*) 'will speed up the reaction more efficiently than manipulating the other'
  write(*,*) 'concentrations.'
 ELSE IF (n_effprod_real .EQ. 1 .AND. n_effreac_real .EQ. 0) THEN
  write(*,*) 'Removing the product released in step', TRIM(indices_prod_char(1)), ' from the reaction mixture'
  write(*,*) 'will speed up the reaction more efficiently than manipulating the other'
  write(*,*) 'concentrations.'
 ELSE IF (n_effprod_real .EQ. 2 .AND. n_effreac_real .GT. 0) THEN
  write(*,*) 'removing the products released in steps', TRIM(indices_prod_char(1)),' and', TRIM(indices_prod_char(2)), &
             ' from the reaction mixture'
  write(*,*) 'will speed up the reaction more efficiently than manipulating the other'
  write(*,*) 'concentrations.'
 ELSE IF (n_effprod_real .EQ. 2 .AND. n_effreac_real .EQ. 0) THEN
  write(*,*) 'Removing the products released in steps', TRIM(indices_prod_char(1)),' and', TRIM(indices_prod_char(2)), &
             ' from the reaction mixture'
  write(*,*) 'will speed up the reaction more efficiently than manipulating the other'
  write(*,*) 'concentrations.'
 ELSE IF (n_effprod .GT. 2 .AND. n_effreac_real .GT. 0) THEN
  write(*,*) 'removing the products released in steps',(TRIM(indices_prod_char(i)),',',i = 1, n_effprod_real-2), &
             TRIM(indices_prod_char(n_effprod_real-1)),' and', TRIM(indices_prod_char(n_effprod_real)),' from the reaction mixture'
  write(*,*) 'will speed up the reaction more efficiently than manipulating the other'
  write(*,*) 'concentrations.'
 ELSE IF (n_effprod .GT. 2 .AND. n_effreac_real .EQ. 0) THEN
  write(*,*) 'Removing the products released in steps',(TRIM(indices_prod_char(i)),',',i = 1, n_effprod_real-2), &
             TRIM(indices_prod_char(n_effprod_real-1)),' and', TRIM(indices_prod_char(n_effprod_real)),' from the reaction mixture'
  write(*,*) 'will speed up the reaction more efficiently than manipulating the other'
  write(*,*) 'concentrations.'
 END IF
 print*,''
END IF

! print the TOF
IF (out_tof) THEN
 IF (out_header) THEN
  print*, '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
  print*, '!!                                                                !!'
  print*, '!!                       THE CALCULATED TOF                       !!'
  print*, '!!                                                                !!'
  print*, '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
  print*,''
 END IF
 write(*,'(A12,ES9.2,A7)') '      TOF = ', TOF, ' s^(-1)'
 TOF = TOF * 3600
 write(*,'(A12,ES9.2,A7)') '          = ', TOF, ' h^(-1)'
 TOF = TOF * 24
 write(*,'(A12,ES9.2,A7)') '          = ', TOF, ' d^(-1)'
 print*,''
END IF

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!! OUTPUT FOR DEBUGGING SECTION !!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! print the energy values after conversion
IF (debug_energy) THEN
 print*, '           Int       TS'
 DO i = 1, steps
  write(*,'(A4,I2,2F10.3)') 'Step', i, IM(i-1), TS(i)
 END DO
 write(*,'(A6,F10.3,/)') 'Final ', IM(steps)
 write(*,'(A6,F10.3,/)') 'DeltaG', deltag
END IF

! print the array rate_indices as columns of the matrix M
IF (debug_rate_indices) THEN
 print*, 'rate constant indices of the partition matrix as in'
 print*, 'Christiansen, J.A., Adv. Catal. 1953, 5, 311'
 DO i = 1, steps
  print*, 'column', i
  DO j = 1, steps
   write(*,steps_dependent_format_I) rate_indices(j,i,:) 
  END DO
 END DO
 print*,''
END IF

! print the array im_indices as columns of the matrix M
IF (debug_im_indices) THEN
 print*, 'intermediate indices of the partition matrix'
 DO i = 1, steps
  print*, 'column', i
  DO j = 1, steps
   write(*,steps_dependent_format_I) im_indices(j,i,:) 
  END DO
 END DO
 print*,''
END IF

! print the array ts_indices as columns of the matrix M
IF (debug_ts_indices) THEN
 print*, 'transition state indices of the partition matrix'
 DO i = 1, steps
  print*, 'column', i
  DO j = 1, steps
   write(*,steps_dependent_format_I) ts_indices(j,i,:) 
  END DO
 END DO
 print*,''
END IF

! print the array im_rem in matrix form
IF (debug_im_rem) THEN
 print*, 'remaining intermediate indices'
 DO i=1,steps
  write(*,steps_dependent_format_I) im_rem(i,:)
 END DO
 print*,''
END IF

! print the array ts_rem in matrix form
IF (debug_ts_rem) THEN
 print*, 'remaining transition state indices'
 DO i=1,steps
  write(*,steps_dependent_format_I) ts_rem(i,:)
 END DO
 print*,''
END IF

! print the array matrix_energy in matrix form
IF (debug_matrix_energy) THEN
 print*, 'the energy matrix'
 DO i = 1, steps
  write(*,steps_dependent_format_E) matrix_energy(i,:)
 END DO
 print*,''
END IF

! map matrix_energy to the arrangements in the published excel sheet
! and print it
IF (debug_matrix_excel) THEN
 print*, 'the energy matrix in the form of Sebastians Excel sheet'
 DO i = 1, steps
  DO j = 1, steps
   matrix_excel(i,j) = matrix_energy(i,j - i + 1 - int((j - steps - i + 1) / steps) * steps)
  END DO
 END DO
 DO i = 1, steps
  write(*,steps_dependent_format_E) matrix_excel(i,:)
 END DO
 print*,''
END IF

! print the array matrix_conc in matrix form
IF (debug_matrix_conc) THEN
 print*, 'the concentration matrix'
 DO i = 1, steps
  write(*,steps_dependent_format_E) matrix_conc(i,:)
 END DO
 print*,''
END IF

! print the array matrix_complete in matrix form
IF (debug_matrix_complete) THEN
 print*, 'the complete matrix'
 DO i = 1, steps
  write(*,steps_dependent_format_E) matrix_complete(i,:)
 END DO
 print*,''
END IF

! print numerator and denominator
IF (debug_numer_denom) THEN
 write(*,'(A21,E10.3)') 'numerator          = ', numerator
 write(*,'(A21,E10.3)') 'denominator        = ', denominator
 print*,''
END IF

! print first term of XTOF_reac and XTOF_prod and second terms of
! respective reactants and products
IF (debug_XTOFC) THEN
 write(*,'(A42,E10.3)') 'exp(-DeltaG)*concproduct_reac/numerator = ', exp(-DeltaG)*concproduct_reac/numerator
 write(*,'(A42,E10.3)') '-concproduct_prod/numerator             = ', -concproduct_prod/numerator
 print*, ''
 print*, '          sum M_a,b,C_reac   sum M_a,b,C_prod'
 DO i = 1, steps
  write(*,'(A5,I2,2E18.3)') 'step ', i, sum_reac(i) / denominator, sum_prod(i) / denominator
 END DO
END IF

! get rid of the arrays
deallocate(indices_reac_char)
deallocate(indices_prod_char)
deallocate(indices_effreac)
deallocate(indices_effprod)
deallocate(TS)
deallocate(IM)
deallocate(TS_kcal)
deallocate(IM_kcal)
deallocate(conc_reac)
deallocate(conc_prod)
deallocate(rate_indices)
deallocate(ts_indices)
deallocate(im_indices)
deallocate(ts_rem)
deallocate(im_rem)
deallocate(matrix_energy)
deallocate(matrix_excel)
deallocate(matrix_conc)
deallocate(matrix_complete)
deallocate(X_TOF_I)
deallocate(X_TOF_TS)
deallocate(sum_reac)
deallocate(sum_prod)
deallocate(X_TOF_reac)
deallocate(X_TOF_prod)
deallocate(reac_exist)
deallocate(prod_exist)
deallocate(conc_reac_string)
deallocate(conc_prod_string)
deallocate(X_reac_string)
deallocate(X_prod_string)

END PROGRAM 
